@@ -40,6 +40,7 @@ sub new {
 		ai_primary_keys => $$req{ai_primary_keys},
 		cols_types => $$req{columns},
 		'+cols_types' => $$req{'+columns'},
+		defaults => $$req{defaults},
 
 
 	 };
@@ -88,6 +89,7 @@ sub set_row_pos{
 		$self->{log}->debug("no data");
 		$found = 0;
 	} elsif ($pos < $self->{row}->{last_row} + 1 && $pos >=0) {
+		$self->{log}->debug("setting current row to an existing row at pos ", $pos);
 		$self->{row}->{pos}= $pos;
 		# $self->{log}->debug("set_row at pos : " . $pos . " pk: " . join(" ", @{ $self->{data}[$pos] }) . " class: " . $self->{rs}->result_class);	
 		#die Dumper( $self->{data}[$pos] );
@@ -125,8 +127,10 @@ sub set_row_pos{
 		 
 		
 	}  elsif ($pos == $self->{row}->{last_row} +1) {
-		$self->{log}->debug("setting current row to undef");
-		$self->{current_row} = undef;
+		#$self->{log}->debug("setting current row to undef");
+		#$self->{current_row} = undef;
+		$self->{log}->debug("setting current row to new row");
+		$self->{current_row} = $self->{new_row};
 		
 	} #else { $found = 0; croak(" position outside rows limits "); }
 	# $self->{log}->debug("set_row_pos current pos: " . $self->{row}->{pos} . " new pos : " . $pos . " last: " . $self->{row}->{last_row} . " count : " . scalar @{ $self->{data}} );
@@ -170,7 +174,11 @@ sub get_field{
 	if (! defined  $row) {
 		 $self->{log}->debug("current row undef because there's no row");
 	} else {
-		$value  = $row->get_column($id);
+		try {
+			$value  = $row->get_column($id);
+		} catch {
+			$self->{log}->error("No column named " . $id  ." in row");
+		}
 	}
 	$self->{log}->debug("get_field " . $id . " value: " . (defined $value ? $value: "undef"));
 	return $value; 
@@ -186,8 +194,8 @@ sub save{
 		$self->{log}->debug(" save new row " );
 		$row = $self->{new_row};
 		#$row->update;
-		$result = $row->insert;
-		$self->{log}->debug(" insert return ". ($result ? " def": " undef"));
+		$done =( defined $row->insert ? 1 : 0);
+		$self->{log}->debug(" insert return ". $done);
 		my @pk_val;
 		for my $pk ($row->primary_columns){
 			my $pkval =  $row->get_column($pk);
@@ -195,9 +203,12 @@ sub save{
 			push @pk_val, $pkval;
 		}
 		push @{$self->{data}}, \@pk_val ;
-		my $last = $self->row_count-1;
-		$self->{row} = {pos => $last, last_row => $last};
-		$self->set_row_pos($last);	
+	# setting the cursor on the newly added row is done in Form->apply 
+		#my $last = $self->row_count-1;
+		# my $last = scalar (@{$self->{data}}) -1;
+		#$self->{log}->debug("last after saved new row: ", $last);
+		#$self->{row} = {pos => $last, last_row => $last};
+		# $self->set_row_pos($last);	
 	
 	} else {
 		$self->{log}->debug(" save at " . $self->{row}->{pos} );
@@ -234,10 +245,26 @@ sub new_row {
 	my $rs =  $self->{rs};
 	my %hash   = map { $_, undef } @{$self->{primary_keys}};
 	#my %hash   = map { $_, undef } @{$self->{cols}};
+	if ($self->{defaults}){
+		my %hd = %{$self->{defaults}};
+		foreach my $k (keys %hd){
+			$hash{$k}= $hd{$k};
+		}
+	
+	}
+	#row->insert is done in save
 	my $row = $rs->new_result(\%hash);
 	$self->{new_row} = $row;
-	$self->{row}->{pos} = $self->{row}->{last_row} + 1;
-	$self->{log}->debug("new_row");
+	#to insure that the new row is at last + 1:
+        if  (defined $self->{row}->{last_row}) {
+		$self->{row}->{pos} = $self->{row}->{last_row} + 1;
+	} else {
+		$self->{row}->{pos}=0;   
+		$self->{row}->{last_row}=-1;
+	}
+
+	$self->{log}->debug("new_row pos:", $self->{row}->{pos}, " last: ", $self->{row}->{last});
+	#$self->{log}->debug( Dumper($row));
 	
 }
 
@@ -443,7 +470,7 @@ Gtk2::Ex::DbLinker::DbcDataManager - a module used by Form and Datasheet that ge
 
 =head1 VERSION
 
-See Version in L<Gtk2::Ex::DbLinker>
+See Version in L<Gtk2::Ex::DbLinker::DbTools>
 
 =head1 SYNOPSIS
 
@@ -624,7 +651,7 @@ Return an array of auto incremented primary key(s) from the underlying result_so
 Any Gk2::Ex::DbLinker questions or problems can be posted to me (rappazf) on my gmail account. 
 
 The current state of the source can be extract using Mercurial from
-L<http://sourceforge.net/projects/gtk2-ex-dblinker/>.
+L<http://sourceforge.net/projects/gtk2-ex-dblinker-dbtools/>.
 
 =head1 AUTHOR
 
