@@ -25,6 +25,7 @@ sub new {
 		$self->{sql}= $$req{sql}; # A hash of SQL related stuff
 		$self->{before_query} = $$req{before_query};
 		$self->{defaults} = $$req{defaults};
+		$self->{ro_fields} = $$req{ro_fields}; #fields from join tables that are not updatable
 
 # an array of auto incremented primary keys
 # and I want to test if it has been set to undef to forcibly set auto_imcrementing to 0
@@ -174,7 +175,7 @@ sub query {
 	
 	};
        	$self->rs_init($rec, undef); 
-       
+        $self->{sql_dump} =  $self->{sql}->{pass_through};
         return scalar @$rec;
 
     } else {  
@@ -202,6 +203,7 @@ sub query {
             $local_sql .= " order by " . $self->{sql}->{order_by};
         }
 	 $self->{log}->debug("local_sql " . $local_sql);
+	 $self->{sql_dump} = $local_sql;
         try {
             $sth = $self->{dbh}->prepare( $local_sql );
                
@@ -215,9 +217,11 @@ sub query {
         try {
             if ( $where_object->{bind_values} ) {
                 $sth->execute( @{$where_object->{bind_values}} );
+            } elsif ( $self->{sql}->{bind_values} ) {
+                $sth->execute( @{$self->{sql}->{bind_values}} ) ;
             } else {
-                $sth->execute ;
-            }
+	    	$sth->execute;
+	    }
         } catch { 
 		confess $self->{dbh}->errstr . " ".  $local_sql;
 	};
@@ -237,7 +241,7 @@ sub query {
 	     push @all_pk_vals, \@key_vals if (@key_vals);
 	   
         }
-	$self->{log}->debug("all_pk_values: ", Dumper (@all_pk_vals));
+	#$self->{log}->debug("all_pk_values: ", Dumper (@all_pk_vals));
 	$self->rs_init(\@all_pk_vals, \@pks);
 
 	#$self->{keyset_size}
@@ -310,7 +314,7 @@ sub _get_rows_from_batch {
 	if ( $self->{sql}->{order_by} ) {
             $local_sql .= " order by " . $self->{sql}->{order_by};
         }
-	$self->{log}->debug($local_sql);
+	#$self->{log}->debug($local_sql);
 
 	my $data;
         try {
@@ -360,6 +364,12 @@ my $done=1;
 		$self->{log}->debug("jumping $fieldname because it's a pk");
             next;
         }
+
+	if (grep (/^$fieldname$/, @{$self->{ro_fields}})){
+		$self->{log}->debug("jumping $fieldname because it's readonly");
+		next;
+	
+	}
 	#if ( defined $widget && ref $widget ne "Gtk2::Label" ) { # Labels are read-only
             push @fieldlist, $fieldname;
 	    my $v =  $self->get_field( $fieldname );
@@ -370,15 +380,20 @@ my $done=1;
     }
     
     my $update_sql;
-    
+   	my ($table) = split (/\s+/, $self->{sql}->{from}, 2); 
     if ( $self->{inserting} ) {
-        $update_sql = "insert into " . $self->{sql}->{from} . " ( " . join( ",", @fieldlist, ) . " )"
-            . " values ( " . "?," x ( @fieldlist - 1 ) . "? )";
+	    #$update_sql = "insert into " . $self->{sql}->{from} . " ( " . join( ",", @fieldlist, ) . " )" . " values ( " . "?," x ( @fieldlist - 1 ) . "? )";
+	   $update_sql = "insert into " . $table . " ( " . join( ",", @fieldlist, ) . " )" . " values ( " . "?," x ( @fieldlist - 1 ) . "? )";
         $self->{log}->debug("inserting ");
     } else {
          $self->{log}->debug("updating ");
 	 # $update_sql = "update " . $self->{sql}->{from} . " set " . join( "=?, ", @fieldlist ) . "=? where " . join( "=? and ", @{$self->{primary_keys}} ) . "=?";
-        $update_sql = "update " . $self->{sql}->{from} . " set " . join( "=?, ", @fieldlist ) . "=? where " . join( "=? and ", @pk ) . "=?";
+	 # changed 01.2016: take the first word of the from value to get the table name
+	 # with the whole from string, problems arise when field names are identical in tables used in the join syntax
+	 # if problems occurs when more then one tables are updated, a new from_update parameters will have to be specified
+
+	
+        $update_sql = "update " . $table . " set " . join( "=?, ", @fieldlist ) . "=? where " . join( "=? and ", @pk ) . "=?";
 	#foreach my $primary_key ( @{$self->{primary_keys}} ) {
 	foreach my $primary_key ( @pk ) {
  		$self->{log}->debug("push on bind_values " . $primary_key . " : " . $self->get_field($primary_key));
@@ -580,6 +595,10 @@ C<primary_keys> : an array ref of the primary key(s) name. With Mysql these are
 
 =item *
 
+C<ro_fields> : an array fef of read only fields from joined tables that are not to be included in update and insert sql statement
+
+=item *
+
 C<ai_primary_key> : an array ref of the auto incremeted primary key name. With Mysql this is corretly detected. You may pass ai_primary_key as undef to indicate that the primary key is not auto incrementing
 
 =item *
