@@ -51,7 +51,7 @@ sub rs_init {
 	$self->{batch_pos} = undef;
 }
 
-#return an AoA of pks values or an A of rows if there is no pks
+# return 1 if there is row data to be get from a new subset of pk values
 sub rs_move {
    my ( $self, $offset, $absolute ) = @_;
   
@@ -82,8 +82,8 @@ sub rs_move {
         if (defined $self->{batch_pos}) {
             if ( $self->{batch_group} != $new_batch_group ) {
                 $self->{batch_group} = $new_batch_group;
-                $self->_fetch_new_batch;
-		$isnewbatch = 1;
+                $isnewbatch = $self->_fetch_new_batch;
+		#$isnewbatch = 1;
             } #else {
 	    
 	     #same batch
@@ -92,8 +92,8 @@ sub rs_move {
 
 	} else {
             $self->{batch_group} = $new_batch_group;
-            $self->_fetch_new_batch;
-	    $isnewbatch = 1;
+            $isnewbatch = $self->_fetch_new_batch;
+	    #$isnewbatch = 1;
         }
         
         $self->{batch_pos} = $new_position - ( $new_batch_group * $self->{batch_size} );
@@ -112,6 +112,7 @@ sub rs_move {
 #This row is an array ref that contains the values for the pk. There is batch_size values unless
 #$self->pkvalues_filter is 1: the pk values are added only once in the array. 
 #$self->pkvalues_filter is 0: all the values are added. 
+#return 1 if there is row data to be get 
 sub _fetch_new_batch {
     
     # Fetches a new set  of pk values ( based on the aperture size )
@@ -120,7 +121,7 @@ sub _fetch_new_batch {
     # Get max value for the loop 
     my $lower = $self->{batch_group} * $self->{batch_size} ;
     my $upper = ( ($self->{batch_group} + 1) * $self->{batch_size} ) - 1;
-    
+     my $isnewbatch=1; 
     
 	my $batch_count = $self->row_count;
 
@@ -138,8 +139,8 @@ sub _fetch_new_batch {
         
         # First, we have to delete anything in $self->{records}
         $self->{records} = ();
-        $self->{log}->debug("query called on an empty recordset");
-	#$data = $self->{records};
+        $self->{log}->debug("_fetch_new_batch called on an empty recordset");
+	$isnewbatch = 0;
     } else {
         
         if ( $upper > $batch_count - 1 ) {
@@ -179,6 +180,7 @@ sub _fetch_new_batch {
         
     }
     $self->{batch} =\@data;
+    return $isnewbatch;
 }
 
 sub get_pkvalues_from_batch {
@@ -232,7 +234,8 @@ sub set_field {
 
 sub get_field {
 my ($self, $fieldname) = @_;
-#die (Dumper $self->{records});
+  # $self->{log}->debug("get_field - name : " . $fieldname . "\nRecords: " . (Dumper $self->{records}));
+  confess ("get_field($fieldname) called on an empty recordset ! Did you forget to call set_row_pos(0) after query ?") unless ($self->{records});
    my $data = $self->{records}[$self->{batch_pos}]->{$fieldname};
    return $data;
 }
@@ -335,6 +338,7 @@ sub set_row_pos{
 	#Dumper(@batch_pk_vals);
          if ( $newbatch) {
 		 #die $pos;
+		 #$self->{log}->debug("set inserting to 0");
 		 $self->{inserting}=0;
 		 my $data =  $self->_get_rows_from_batch($self->get_pkvalues_from_batch) ;
 		 #die (ref $data);
@@ -361,7 +365,7 @@ sub get_field_type {
 sub new_row {
    
     my $self = shift;
-    # $self->{log}->debug("new row rows before new are : ", Dumper($self->rs_get_rows));
+    #$self->{log}->debug("new row called : set inserting flag");
     my $newposition = $self->row_count; # No need to add one, as the array starts at zero.
 
 	$self->rs_move( undef, $newposition);
