@@ -91,6 +91,7 @@ sub new {
 	   # peut redefinir auto_incrementing avec mysql
 	   $self->_init_pks;
    	}
+	$self->{log}->debug("primary_keys array : ", join( " ", @{$self->{primary_keys}}), "\n");
 	$self->{log}->debug("pks: ", join( " ", $self->get_primarykeys), "\n");
  	$self->{primary_keys} = $self->{ai_primary_key} if (defined $self->{ai_primary_key});
 	 $self->{log}->debug("auto_incrementing: " . 	$self->{auto_incrementing});
@@ -108,8 +109,11 @@ sub new {
 
 }
 
-#comment conserver le mecanisme de lecture par paquet avec l'utilisation de -in dans sql::Abstract ... a suivre
-#keyset est un array ref, of arrays, each reccord fetch by query is remembered by an array of the primary key(s) value(s). Most of the time, the array will have one element.
+#Replace for the function duration the -where parameter setted in the constructor by the arg received in the call
+#Get the pk values for the rows corresponding to this -where parameter
+#Build a new recordset object using these values and restore the original -where and -col parameter.
+#
+#keyset is an array ref, of arrays, each reccord fetch by query is remembered by an array of the primary key(s) value(s). Most of the time, the array will have one element.
 #If query return 10 records, keyset will refer to an array of 10 arrays
 sub query {
 	 my ( $self,  $where ) = @_;
@@ -241,7 +245,7 @@ sub save {
     my %new_values;
 	#my @fieldlist = ();
 	#my @bind_values = ();
-	
+    $self->{log}->debug(" save: inserting is " . $self->{inserting});	
     #$href is used to change a field's value when the field is included in a composed primary keys. 
     #The array @pk holds the field's name of the primary keys since ->get_primarykeys return these fields even if auto_incrementing is 0
     #The if test in the foreach loop fails and the values of the primary key fields are not added in the bind_values array therefore.
@@ -258,12 +262,16 @@ sub save {
 		#push @fieldlist, $k;
 		$new_values{$k} = $href->{$k};
 	}
-    	@pk = $self->get_primarykeys;
+	# @pk = $self->get_primarykeys;
     } else {
-     	@pk  = $self->get_autoinc_primarykeys;
+	 # @pk  = $self->get_autoinc_primarykeys;
+	#avec la table abo et la jointure sur jrnabt, la mise a jour ne marche pas car get_autoinc_primarykey retroune un vecteur vide
+	#noabt est alors traite comme un champ normal
+	#prendre toujours le resultat de get_primarykeys corrige ce probleme
      }
-   #$self->{log}->debug("pk: " . join(" ", @pk));
-   #$self->{log}->debug("primary_keys: " . join(" ", @{$self->{primary_keys}}));
+     @pk =  $self->get_primarykeys;
+   $self->{log}->debug("pk: " . join(" ", @pk));
+   $self->{log}->debug("primary_keys: " . join(" ", @{$self->{primary_keys}}));
 
 my %pk_values;
 my $table = $self->_get_tablename;     
@@ -276,7 +284,7 @@ my ($value, $stm, @bind);
    
 	
 	#if ( $sql_fieldname ~~ @pk) {
-	if (grep (/^$fieldname$/, @pk)) {
+	if (grep (/\b$fieldname\b/, @pk)) {
 		$self->{log}->debug("jumping $fieldname because it's a pk");
             next;
         }
@@ -351,6 +359,7 @@ my ($value, $stm, @bind);
         
     }
     $self->{inserting} = 0;
+    $self->{log}->debug("turn off inserting flag");
     return $done;
 }
 
